#!/usr/bin/env python2

import espressopp
import read_config_file as config
from espressopp import Real3D

########################################################################
# 1. specification of the main simulation parameters                   #
########################################################################
Npart              = 32768
L                  = 33.9
box                = (L, L, L)
r_cutoff           = 2.5
skin               = 0.4
temperature        = 1.0
dt                 = 0.005
epsilon            = 1.0
sigma              = 1.0

nloops       = 100
isteps       = 100

# print ESPResSo++ version and compile info
print espressopp.Version().info()
# print simulation parameters (useful to have them in a log file)
print "Npart              = ", Npart
print "L                  = ", L
print "box                = ", box 
print "r_cutoff           = ", r_cutoff
print "skin               = ", skin
print "temperature        = ", temperature
print "dt                 = ", dt
print "epsilon            = ", epsilon
print "sigma              = ", sigma
print "nloops       = ", nloops
print "isteps       = ", isteps

########################################################################
# 2. setup of the system, random number geneartor and parallelisation  #
########################################################################

system             = espressopp.System()
system.rng         = espressopp.esutil.RNG()
system.bc          = espressopp.bc.OrthorhombicBC(system.rng, box)
system.skin        = skin
NCPUs              = espressopp.MPI.COMM_WORLD.size
nodeGrid           = espressopp.tools.decomp.nodeGrid(NCPUs)
cellGrid           = espressopp.tools.decomp.cellGrid(box, nodeGrid, r_cutoff, skin)
system.storage     = espressopp.storage.DomainDecomposition(system, nodeGrid, cellGrid)

print "NCPUs              = ", NCPUs
print "nodeGrid           = ", nodeGrid
print "cellGrid           = ", cellGrid

########################################################################
# 3. setup of the integrator and simulation ensemble                   #
########################################################################

integrator     = espressopp.integrator.VelocityVerlet(system)
integrator.dt  = dt
thermostat = espressopp.integrator.LangevinThermostat(system)
thermostat.gamma = 1.0
thermostat.temperature = temperature
integrator.addExtension(thermostat)

## steps 2. and 3. could be short-cut by the following expression:
## system, integrator = espressopp.standard_system.Default(box, warmup_cutoff, skin, dt, temperature)

########################################################################
# 4. adding the particles                                              #
########################################################################

print "adding ", Npart, " particles to the system ..." 
id = config.id
x = config.x
y = config.y
z = config.z
vx = config.vx
vy = config.vy
vz = config.vz
type = config.type

props = ['id', 'pos', 'v', 'type', 'mass']
allParticles = []
for pid in range(Npart):
  allParticles.append([id[pid], Real3D(x[pid], y[pid], z[pid]), Real3D(vx[pid], vy[pid], vz[pid]), type[pid], 1.0])

system.storage.addParticles(allParticles, *props) 
system.storage.decompose()

########################################################################
# 5. setting up interaction potential for the simulation            #
########################################################################

verletlist  = espressopp.VerletList(system, r_cutoff)
interaction = espressopp.interaction.VerletListLennardJones(verletlist)
# use a Lennard-Jones potential between 2 particles of type 0 
# the potential is automatically shifted so that U(r=cutoff) = 0.0
# if the potential should not be shifted set shift=0.0
potLJ = espressopp.interaction.LennardJones(epsilon=epsilon, sigma=sigma, cutoff=r_cutoff, shift=0.0)
interaction.setPotential(type1=0, type2=0, potential=potLJ)
system.addInteraction(interaction)

########################################################################
# 6. running the simulation                                   #
########################################################################
print "starting simulation ..."
# print inital status information
espressopp.tools.analyse.info(system, integrator)
for step in range(nloops):
  integrator.run(isteps)
  espressopp.tools.analyse.info(system, integrator)
  
  # sampling configuration
  #folded config
  #filename1 = "config/lennard_jones_fluid_%0i.xyz" % step
  #espressopp.tools.writexyz(filename1, system, velocities = True, unfolded = False)
  #unfolded config
  #filename2 = "config_unfolded/lennard_jones_fluid_%0i.xyz" % step
  #espressopp.tools.writexyz(filename2, system, velocities = True, unfolded = True)
  #pdbs
  #filename3 = "pdbs/lennard_jones_fluid_%0i.pdb" % step
  #espressopp.tools.pdbwrite(filename3, system, molsize=Npart)
  
print "simulation finished"

########################################################################
# 7. writing configuration to file                                     #
########################################################################

# write folded xyz coordinates and particle velocities into a file
# format of xyz file is:
# first line      : number of particles
# second line     : box_Lx, box_Ly, box_Lz
# all other lines : ParticleID  ParticleType  x_pos  y_pos  z_pos  x_vel  y_vel  z_vel 
filename = "lennard_jones_fluid_%0i.xyz" % integrator.step
print "writing final configuration file ..." 
espressopp.tools.writexyz(filename, system, velocities = True, unfolded = False)

# also write a PDB file which can be used to visualize configuration with VMD
print "writing pdb file ..."
filename = "lennard_jones_fluid_%0i.pdb" % integrator.step
espressopp.tools.pdbwrite(filename, system, molsize=Npart)

print "finished."
